!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #################
      MODULE MODD_IDEAL_n
!     #################
!
!!****  *MODD_IDEAL_n - declaration of surface parameters for an inland water surface
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_TYPE_DATE_SURF
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE IDEAL_t
!
! Time-step:
!
  REAL                                  :: XTSTEP  ! time step
!
  REAL                                  :: XOUT_TSTEP  ! output writing time step
!
!
!
END TYPE IDEAL_t

TYPE(IDEAL_t), ALLOCATABLE, TARGET, SAVE :: IDEAL_MODEL(:)

REAL, POINTER :: XTSTEP=>NULL()
!$OMP THREADPRIVATE(XTSTEP)
REAL, POINTER :: XOUT_TSTEP=>NULL()
!$OMP THREADPRIVATE(XOUT_TSTEP)

CONTAINS

SUBROUTINE IDEAL_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_IDEAL_N:IDEAL_GOTO_MODEL',0,ZHOOK_HANDLE)
XTSTEP=>IDEAL_MODEL(KTO)%XTSTEP
XOUT_TSTEP=>IDEAL_MODEL(KTO)%XOUT_TSTEP
IF (LHOOK) CALL DR_HOOK('MODD_IDEAL_N:IDEAL_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE IDEAL_GOTO_MODEL

SUBROUTINE IDEAL_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_IDEAL_N:IDEAL_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(IDEAL_MODEL(KMODEL))
IDEAL_MODEL(:)%XTSTEP=0.
IDEAL_MODEL(:)%XOUT_TSTEP=0.
IF (LHOOK) CALL DR_HOOK("MODD_IDEAL_N:IDEAL_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE IDEAL_ALLOC

SUBROUTINE IDEAL_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_IDEAL_N:IDEAL_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(IDEAL_MODEL)) DEALLOCATE(IDEAL_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_IDEAL_N:IDEAL_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE IDEAL_DEALLO

END MODULE MODD_IDEAL_n
