!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ##################
      MODULE MODD_SURF_ATM_GRID_n
!     ##################
!
!!****  *MODD_SURF_ATM_GRID - declaration of SURF_ATM grid
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson  *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SURF_ATM_GRID_t
!-------------------------------------------------------------------------------
!
! Grid definition
!
  CHARACTER(LEN=10)               :: CGRID       ! grid type
!                                              ! "NONE        " : no grid computations
!                                              ! "CONF PROJ   " : conformal projection
!
  REAL, POINTER,     DIMENSION(:) :: XGRID_PAR   ! lits of parameters used to define the grid
!                                              ! (depends on value of CGRID)
  REAL, POINTER,     DIMENSION(:) :: XGRID_FULL_PAR   ! lits of parameters used to define the grid
!                                                     ! (depends on value of CGRID)
  INTEGER                         :: NGRID_PAR   ! size of XGRID_PAR
!
  INTEGER, POINTER, DIMENSION(:,:) :: NNEAR
!-------------------------------------------------------------------------------
!
! General surface parameters:
!
  REAL, POINTER, DIMENSION(:) :: XLAT        ! latitude (degrees +North)               (-)
  REAL, POINTER, DIMENSION(:) :: XLON        ! longitude (degrees +East)               (-)
  REAL, POINTER, DIMENSION(:) :: XMESH_SIZE  ! mesh size                               (m2)
  REAL, POINTER, DIMENSION(:) :: XJPDIR      ! heading of J direction (deg from N clockwise)
!-------------------------------------------------------------------------------
!

END TYPE SURF_ATM_GRID_t

TYPE(SURF_ATM_GRID_t), ALLOCATABLE, TARGET, SAVE :: SURF_ATM_GRID_MODEL(:)

 CHARACTER(LEN=10), POINTER :: CGRID=>NULL()
!$OMP THREADPRIVATE(CGRID)
REAL, POINTER,     DIMENSION(:) :: XGRID_PAR=>NULL()
!$OMP THREADPRIVATE(XGRID_PAR)
INTEGER, POINTER,     DIMENSION(:,:) :: NNEAR=>NULL()
!$OMP THREADPRIVATE(NNEAR)
REAL, POINTER,     DIMENSION(:) :: XGRID_FULL_PAR=>NULL()
!$OMP THREADPRIVATE(XGRID_FULL_PAR)
INTEGER, POINTER :: NGRID_PAR=>NULL()
!$OMP THREADPRIVATE(NGRID_PAR)
REAL, POINTER, DIMENSION(:) :: XLAT=>NULL()
!$OMP THREADPRIVATE(XLAT)
REAL, POINTER, DIMENSION(:) :: XLON=>NULL()
!$OMP THREADPRIVATE(XLON)
REAL, POINTER, DIMENSION(:) :: XMESH_SIZE=>NULL()
!$OMP THREADPRIVATE(XMESH_SIZE)
REAL, POINTER, DIMENSION(:) :: XJPDIR=>NULL()
!$OMP THREADPRIVATE(XJPDIR)

CONTAINS

SUBROUTINE SURF_ATM_GRID_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
SURF_ATM_GRID_MODEL(KFROM)%XGRID_PAR=>XGRID_PAR
SURF_ATM_GRID_MODEL(KFROM)%NNEAR=>NNEAR
SURF_ATM_GRID_MODEL(KFROM)%XGRID_FULL_PAR=>XGRID_FULL_PAR
SURF_ATM_GRID_MODEL(KFROM)%XLAT=>XLAT
SURF_ATM_GRID_MODEL(KFROM)%XLON=>XLON
SURF_ATM_GRID_MODEL(KFROM)%XMESH_SIZE=>XMESH_SIZE
SURF_ATM_GRID_MODEL(KFROM)%XJPDIR=>XJPDIR
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_SURF_ATM_GRID_N:SURF_ATM_GRID_GOTO_MODEL',0,ZHOOK_HANDLE)
CGRID=>SURF_ATM_GRID_MODEL(KTO)%CGRID
XGRID_PAR=>SURF_ATM_GRID_MODEL(KTO)%XGRID_PAR
NNEAR=>SURF_ATM_GRID_MODEL(KTO)%NNEAR
XGRID_FULL_PAR=>SURF_ATM_GRID_MODEL(KTO)%XGRID_FULL_PAR
NGRID_PAR=>SURF_ATM_GRID_MODEL(KTO)%NGRID_PAR
XLAT=>SURF_ATM_GRID_MODEL(KTO)%XLAT
XLON=>SURF_ATM_GRID_MODEL(KTO)%XLON
XMESH_SIZE=>SURF_ATM_GRID_MODEL(KTO)%XMESH_SIZE
XJPDIR=>SURF_ATM_GRID_MODEL(KTO)%XJPDIR
IF (LHOOK) CALL DR_HOOK('MODD_SURF_ATM_GRID_N:SURF_ATM_GRID_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE SURF_ATM_GRID_GOTO_MODEL

SUBROUTINE SURF_ATM_GRID_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_GRID_N:SURF_ATM_GRID_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(SURF_ATM_GRID_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(SURF_ATM_GRID_MODEL(J)%XGRID_PAR)
  NULLIFY(SURF_ATM_GRID_MODEL(J)%NNEAR)
  NULLIFY(SURF_ATM_GRID_MODEL(J)%XGRID_FULL_PAR)
  NULLIFY(SURF_ATM_GRID_MODEL(J)%XLAT)
  NULLIFY(SURF_ATM_GRID_MODEL(J)%XLON)
  NULLIFY(SURF_ATM_GRID_MODEL(J)%XMESH_SIZE)
  NULLIFY(SURF_ATM_GRID_MODEL(J)%XJPDIR)
ENDDO
SURF_ATM_GRID_MODEL(:)%CGRID=' '
SURF_ATM_GRID_MODEL(:)%NGRID_PAR=0
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_GRID_N:SURF_ATM_GRID_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE SURF_ATM_GRID_ALLOC

SUBROUTINE SURF_ATM_GRID_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_GRID_N:SURF_ATM_GRID_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(SURF_ATM_GRID_MODEL)) DEALLOCATE(SURF_ATM_GRID_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_SURF_ATM_GRID_N:SURF_ATM_GRID_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE SURF_ATM_GRID_DEALLO

END MODULE MODD_SURF_ATM_GRID_n
