!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ################
      MODULE MODD_TEB_CANOPY_n
!     ################
!
!!****  *MODD_TEB_CANOPY_n - declaration of surface parameters for urban canopy
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/2006
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE TEB_CANOPY_t
!
  INTEGER                       :: NLVL ! number      of levels in canopy
  REAL, POINTER, DIMENSION(:,:) :: XZ   ! height of middle of each level grid   (m)
  REAL, POINTER, DIMENSION(:,:) :: XU   ! wind        at each level in canopy   (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XT   ! temperature at each level in canopy   (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XQ   ! humidity    at each level in canopy   (kg/m3)
  REAL, POINTER, DIMENSION(:,:) :: XTKE ! Tke         at each level in canopy   (m2/s2)
  REAL, POINTER, DIMENSION(:,:) :: XLMO ! Monin-Obhukov length                  (m)
  REAL, POINTER, DIMENSION(:,:) :: XLM  ! Mixing lentgh                         (m)
  REAL, POINTER, DIMENSION(:,:) :: XLEPS! Dissipative length                    (m)
  REAL, POINTER, DIMENSION(:,:) :: XP   ! pressure    at each level in canopy   (kg/m3)
!
  REAL, POINTER, DIMENSION(:,:) :: XDZ  ! depth       of each level in canopy   (m)
  REAL, POINTER, DIMENSION(:,:) :: XZF  ! height of bottom of each level grid   (m)
  REAL, POINTER, DIMENSION(:,:) :: XDZF ! depth between  each level in canopy   (m)
!
END TYPE TEB_CANOPY_t

TYPE(TEB_CANOPY_t), ALLOCATABLE, TARGET, SAVE :: TEB_CANOPY_MODEL(:)

INTEGER, POINTER :: NLVL=>NULL()
!$OMP THREADPRIVATE(NLVL)
REAL, POINTER, DIMENSION(:,:) :: XZ=>NULL()
!$OMP THREADPRIVATE(XZ)
REAL, POINTER, DIMENSION(:,:) :: XU=>NULL()
!$OMP THREADPRIVATE(XU)
REAL, POINTER, DIMENSION(:,:) :: XT=>NULL()
!$OMP THREADPRIVATE(XT)
REAL, POINTER, DIMENSION(:,:) :: XQ=>NULL()
!$OMP THREADPRIVATE(XQ)
REAL, POINTER, DIMENSION(:,:) :: XTKE=>NULL()
!$OMP THREADPRIVATE(XTKE)
REAL, POINTER, DIMENSION(:,:) :: XLMO=>NULL()
!$OMP THREADPRIVATE(XLMO)
REAL, POINTER, DIMENSION(:,:) :: XLM=>NULL()
!$OMP THREADPRIVATE(XLM)
REAL, POINTER, DIMENSION(:,:) :: XLEPS=>NULL()
!$OMP THREADPRIVATE(XLEPS)
REAL, POINTER, DIMENSION(:,:) :: XP=>NULL()
!$OMP THREADPRIVATE(XP)
REAL, POINTER, DIMENSION(:,:) :: XDZ=>NULL()
!$OMP THREADPRIVATE(XDZ)
REAL, POINTER, DIMENSION(:,:) :: XZF=>NULL()
!$OMP THREADPRIVATE(XZF)
REAL, POINTER, DIMENSION(:,:) :: XDZF=>NULL()
!$OMP THREADPRIVATE(XDZF)

CONTAINS

SUBROUTINE TEB_CANOPY_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
TEB_CANOPY_MODEL(KFROM)%XZ=>XZ
TEB_CANOPY_MODEL(KFROM)%XU=>XU
TEB_CANOPY_MODEL(KFROM)%XT=>XT
TEB_CANOPY_MODEL(KFROM)%XQ=>XQ
TEB_CANOPY_MODEL(KFROM)%XTKE=>XTKE
TEB_CANOPY_MODEL(KFROM)%XLMO=>XLMO
TEB_CANOPY_MODEL(KFROM)%XLM=>XLM
TEB_CANOPY_MODEL(KFROM)%XLEPS=>XLEPS
TEB_CANOPY_MODEL(KFROM)%XP=>XP
TEB_CANOPY_MODEL(KFROM)%XDZ=>XDZ
TEB_CANOPY_MODEL(KFROM)%XZF=>XZF
TEB_CANOPY_MODEL(KFROM)%XDZF=>XDZF
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_TEB_CANOPY_N:TEB_CANOPY_GOTO_MODEL',0,ZHOOK_HANDLE)
NLVL=>TEB_CANOPY_MODEL(KTO)%NLVL
XZ=>TEB_CANOPY_MODEL(KTO)%XZ
XU=>TEB_CANOPY_MODEL(KTO)%XU
XT=>TEB_CANOPY_MODEL(KTO)%XT
XQ=>TEB_CANOPY_MODEL(KTO)%XQ
XTKE=>TEB_CANOPY_MODEL(KTO)%XTKE
XLMO=>TEB_CANOPY_MODEL(KTO)%XLMO
XLM=>TEB_CANOPY_MODEL(KTO)%XLM
XLEPS=>TEB_CANOPY_MODEL(KTO)%XLEPS
XP=>TEB_CANOPY_MODEL(KTO)%XP
XDZ=>TEB_CANOPY_MODEL(KTO)%XDZ
XZF=>TEB_CANOPY_MODEL(KTO)%XZF
XDZF=>TEB_CANOPY_MODEL(KTO)%XDZF
IF (LHOOK) CALL DR_HOOK('MODD_TEB_CANOPY_N:TEB_CANOPY_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE TEB_CANOPY_GOTO_MODEL

SUBROUTINE TEB_CANOPY_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_TEB_CANOPY_N:TEB_CANOPY_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(TEB_CANOPY_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(TEB_CANOPY_MODEL(J)%XZ)
  NULLIFY(TEB_CANOPY_MODEL(J)%XU)
  NULLIFY(TEB_CANOPY_MODEL(J)%XT)
  NULLIFY(TEB_CANOPY_MODEL(J)%XQ)
  NULLIFY(TEB_CANOPY_MODEL(J)%XTKE)
  NULLIFY(TEB_CANOPY_MODEL(J)%XLMO)
  NULLIFY(TEB_CANOPY_MODEL(J)%XLM)
  NULLIFY(TEB_CANOPY_MODEL(J)%XLEPS)
  NULLIFY(TEB_CANOPY_MODEL(J)%XP)
  NULLIFY(TEB_CANOPY_MODEL(J)%XDZ)
  NULLIFY(TEB_CANOPY_MODEL(J)%XZF)
  NULLIFY(TEB_CANOPY_MODEL(J)%XDZF)
ENDDO
TEB_CANOPY_MODEL(:)%NLVL=0
IF (LHOOK) CALL DR_HOOK("MODD_TEB_CANOPY_N:TEB_CANOPY_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE TEB_CANOPY_ALLOC

SUBROUTINE TEB_CANOPY_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_TEB_CANOPY_N:TEB_CANOPY_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(TEB_CANOPY_MODEL)) DEALLOCATE(TEB_CANOPY_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_TEB_CANOPY_N:TEB_CANOPY_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE TEB_CANOPY_DEALLO

END MODULE MODD_TEB_CANOPY_n
